/* This code was developed by Ettore Trainiti, then 
   modified and commented by Paolo Falcarin */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

/* Define 'student' as an alias
for the new 'struct stud' type*/
typedef struct stud{
	int id;
	char *name;
	char *surname;
	struct stud * next;
} student;
/* insert a student in the beginning of the list*/

void add_student(student ** head,int temp_id,char name[],char surname[])
{
/* allocate memory for the new variable of type student */
	student *new_student =(student *) malloc(sizeof(student));
	if (new_student==NULL) {
		printf("Out of Memory");
		return;
    }
    new_student->id = temp_id;
    new_student->name = (char *) malloc(strlen(name)+1);
    new_student->surname = (char *) malloc(strlen(surname)+1);
    new_student->next = NULL;
    strcpy(new_student->name, name);
    strcpy(new_student->surname, surname);
	new_student->next=*head;
	*head=new_student;
}

/* insertion sort **/
void insertionSort(student ** head, int n)
{
    int i, j ;
    student ** v = head;
    student * key;
    /*additional code helping in debugging*/
    printf("\nInitial array:");
        for (int k=0; k<n; k++ )
           printf("%d ",v[k]->id); 
	for(j=1; j<n; j++)  {
      	i = j - 1 ;
		key = v[j];
		while ( i >= 0 && (v[i]->id > key->id) ) {
			v[i+1] = v[i] ;
			i-- ;
		}
		v[i+1] = key ;
		/*additional code helping in debugging*/
	    printf("\nAfter iteration %d:",j);
        for (int k=0; k<n; k++ )
           printf("%d ",v[k]->id); 
    }
}

void sort_student(student ** head)
{
	int howmany=0;
	//This "for" counts the number of nodes in the list
	for(student * node = *head; node!=NULL; node= node->next)
		howmany++;
	student ** array = (student **) malloc (howmany*sizeof(student *));
	
    student * node = *head;
    for (int i=0 ; i<howmany; i++, node= node->next)
        array[i] = node;
    printf("There are %d elements\n",howmany);
	insertionSort(array, howmany);
	
    /*rebuild list from array */
	for (int i=0 ; i<howmany-1; i++){
        array[i]->next = array[i+1];
    }
    array[howmany-1]->next = NULL;
    *head = array[0];
    free(array);
}

/*remove linked list from memory*/
void free_list(student * list) {
 student *p;
 while(list != NULL) {
  p = list->next;
  free(list);
  list = p;
 }
}

void get_student_data(student ** head)
{
	int temp_id=0;
	char name[20];
	char surname[20];
    printf("\n\n\t\t\t Add student\n\n");
	printf("Student id : ");
	scanf("%d",&temp_id);
	printf("Student name : ");
	scanf("%s",name);
	printf("Student surname : ");
	scanf("%s",surname);
	add_student(head,temp_id,name,surname);
	return;
}

int id_removing()
{
	int id_to_remove;
	printf("Insert the id user of the student you want to delete : ");
	scanf("%d",&id_to_remove);
	return id_to_remove;
}

void remove_student(student ** head,int id_to_remove)
{
	//Temp pointers declaration
	student * temp_head;
	student * previous_element;
	temp_head=*head;
	previous_element=*head;
	//Delete procedure
	if (temp_head==NULL){
		printf("\nList is empty.\n");
		getchar();
		return;
	}
	//Particular Case -> Delete the first student of the list
	if(temp_head->id==id_to_remove){
		free(temp_head->name);
		free(temp_head->surname);
		*head=temp_head->next;
		free(temp_head);
		return;
	}
	//Normal procedure
	while(temp_head->id!=id_to_remove && temp_head->next!=NULL){
		previous_element=temp_head;
		temp_head=temp_head->next;
	}
	if(temp_head->id==id_to_remove){
		previous_element->next=temp_head->next;
		free(temp_head->name);
		free(temp_head->surname);
		free(temp_head);
		return;
	}
	printf("\nStudent not found\n");
	getchar();
	return;
}

void print_list(student ** head)
{
 	if(*head==NULL){
		printf("List is empty!\n");
		return;
	}
	student * temp_head;
	temp_head=*head;
	while(temp_head!=NULL){
		printf("Student ID : %d\n",temp_head->id);
		printf("Student name : %s\n",temp_head->name);
		printf("Student surname : %s\n",temp_head->surname);
		printf("\n");
		temp_head=temp_head->next;
	}
	getchar();
}

/* This function reads the file from the same 
folder of the program: insert "students.txt" to use
the example */
void readFile(student ** head)
{
	FILE *file;
	int temp_id;
	char c;
	char file_name_buffer[20];
	char name_buffer[20];
	char surname_buffer[20];
	char line[30];
	printf("Insert the name of the file you want to read : ");
	scanf("%s",file_name_buffer);
	file = fopen(file_name_buffer,"r");
	/*With fgets reads a string 'line' of at most 30 chars
    from the file 'file' */
	while(fgets(line,30,file)!=NULL){
        /* sscanf extract variables from string 'line' */
        sscanf(line,"%d %s %s",&temp_id,name_buffer,surname_buffer);
		add_student(head,temp_id,name_buffer,surname_buffer);
	}
	fclose(file);
	return;	
}

void saveFile(student ** head)
{
	FILE *file_pointer;
	char file_name_buffer[20];
	student * temp_head;
	temp_head=*head;
	printf("Insert the name of the file where you want the program save the data : ");
	scanf("%s",file_name_buffer);
	file_pointer=fopen(file_name_buffer,"w");
	while(temp_head!=NULL){
		fprintf(file_pointer,"%d %s %s\n",temp_head->id,temp_head->name,temp_head->surname);
		temp_head=temp_head->next;	
	}
	fclose(file_pointer);
	return;	
}
/*list of students */
student * head=NULL;

int main()
{
	int choice=0;
	do{
    	printf("\n\n\t\t Main Menu\n\n");
		printf("1) Add a student\n");
		printf("2) Remove a student\n");
		printf("3) Print the student list\n");
		printf("4) Sort by ID \n");
		printf("5) Read the students list from a text file\n");
		printf("6) Save the students list in a text file\n");
		printf("7) Exit");
		printf("\n");
		printf("Choose an option : ");
		scanf("%d",&choice);
		switch(choice){
			case 1 : get_student_data(&head); 				break;
			case 2 : remove_student(&head,id_removing()); 	break;
			case 3 : print_list(&head);     				break;
			case 4 : sort_student(&head);   				break;
			case 5 : readFile(&head);  				        break;
			case 6 : saveFile(&head);    				break;
		}
	}while(choice!=7);
   	printf("\nGoodbye\n");
	/*This deletes from memory the whole list*/
	free_list(head);
	getchar();
	return 0;
}
